
USE AdventureWorks
GO

CREATE TABLE EmployeeHID
(
  OrgNode hierarchyid,
  OrgLevel AS OrgNode.GetLevel(),
  	[EmployeeID] [int] NOT NULL,
	[NationalIDNumber] [nvarchar](15) NOT NULL,
	[ContactID] [int] NOT NULL,
	[LoginID] [nvarchar](256) NOT NULL,
	[ManagerID] [int] NULL,
	[Title] [nvarchar](50) NOT NULL,
	[BirthDate] [datetime] NOT NULL,
	[MaritalStatus] [nchar](1) NOT NULL,
	[Gender] [nchar](1) NOT NULL,
	[HireDate] [datetime] NOT NULL,
	[SalariedFlag] [dbo].[Flag] NOT NULL,
	[VacationHours] [smallint] NOT NULL,
	[SickLeaveHours] [smallint] NOT NULL,
	[CurrentFlag] [dbo].[Flag] NOT NULL,
	[rowguid] [uniqueidentifier] NOT NULL,
	[ModifiedDate] [datetime] NOT NULL,
CONSTRAINT PK_EmployeeHID_OrgNode
  PRIMARY KEY CLUSTERED (OrgNode)
)

-- create temp table for number of children
CREATE TABLE #Children 
   (
    EmployeeId int,
    ManagerID int,
    Num int
);
GO

CREATE CLUSTERED INDEX tmpind ON #Children(ManagerID, EmployeeID); 
GO

-- populate the temp table
INSERT #Children (EmployeeID, ManagerID, Num)
SELECT EmployeeID, ManagerID,
  ROW_NUMBER() OVER (PARTITION BY ManagerID ORDER BY ManagerID) 
FROM EmployeeAdj ;
GO

SELECT * FROM #Children ORDER BY ManagerID, Num;
GO

-- populate the NewOrg table
-- uses GetRoot and ToString
WITH paths(path, EmployeeID) 
AS (
-- This section provides the value for the root of the hierarchy
SELECT hierarchyid::GetRoot() AS OrgNode, EmployeeID 
FROM #Children AS C 
WHERE ManagerID IS NULL 

UNION ALL 
-- This section provides values for all nodes except the root
SELECT 
CAST(p.path.ToString() + CAST(C.Num AS varchar(30)) + '/' AS hierarchyid), 
C.EmployeeID
FROM #Children AS C 
JOIN paths AS p 
   ON C.ManagerID = P.EmployeeID 
)
INSERT EmployeeHID (OrgNode, O.EmployeeID, O.NationalIDNumber, O.ContactID, O.LoginID, O.ManagerID, O.Title, O.BirthDate, O.MaritalStatus, O.Gender, O.HireDate, O.SalariedFlag, O.VacationHours, O.SickLeaveHours, O.CurrentFlag, O.rowguid, O.ModifiedDate)
SELECT P.path, O.EmployeeID, O.NationalIDNumber, O.ContactID, O.LoginID, O.ManagerID, O.Title, O.BirthDate, O.MaritalStatus, O.Gender, O.HireDate, O.SalariedFlag, O.VacationHours, O.SickLeaveHours, O.CurrentFlag, O.rowguid, O.ModifiedDate
FROM EmployeeAdj AS O 
JOIN Paths AS P 
   ON O.EmployeeID = P.EmployeeID ;
GO

DROP TABLE #Children
DROP TABLE EmployeeHID

SELECT * FROM EmployeeAdj

SELECT * FROM EmployeeHID

CREATE UNIQUE INDEX EmployeeHID_Lvl  ON EmployeeHID(OrgLevel, OrgNode) ;
GO

alter table EmployeeHID drop column ManagerID