
USE AdventureWorks
GO

-- procedure to insert nodes
-- this procedure will always insert after the last sibling
CREATE PROC AddEmp(@mgrid int, @empid int, @e_name varchar(20), @title varchar(20)) 
AS 
BEGIN
   DECLARE @mgr_hid hierarchyid, @last_child hierarchyid, @hid hierarchyid

   
   BEGIN TRANSACTION
      IF @mgrid IS NULL
          SET @hid = hierarchyid::GetRoot();
      ELSE
      BEGIN
			SELECT @mgr_hid  = OrgNode 
			FROM HumanResources.EmployeeOrg WITH (UPDLOCK)
			WHERE EmployeeID = @mgrid
			
			SELECT @last_child = max(OrgNode) 
			FROM HumanResources.EmployeeOrg 
			WHERE OrgNode.GetAncestor(1) = @mgr_hid;
			
			SET @hid = @mgr_hid.GetDescendant(@last_child, NULL)
      END

      INSERT HumanResources.EmployeeOrg (OrgNode, EmployeeID, EmpName, Title)
      VALUES(@hid, @empid, @e_name, @title)
   COMMIT
END ;
GO

-- add employees
EXEC AddEmp NULL, 6, 'David', 'Marketing Manager' ;
EXEC AddEmp 6, 46, 'Sariya', 'Marketing Specialist' ;  
EXEC AddEmp 6, 271, 'John', 'Marketing Specialist' ;
EXEC AddEmp 6, 119, 'Jill', 'Marketing Specialist' ;
EXEC AddEmp 46, 269, 'Wanida', 'Marketing Assistant' ;
EXEC AddEmp 271, 272, 'Mary', 'Marketing Assistant' ;
GO

-- query hierarchy
SELECT OrgNode.ToString() AS Text_OrgNode, 
OrgNode, OrgLevel, EmployeeID, EmpName, Title 
FROM HumanResources.EmployeeOrg ;
GO


 
